#!/bin/sh

dir_this="/share/Apps/LinuxUtils"
dir_trmd="$dir_this/transmission"
bak_path="/nmt/apps/backup.ltu"

func_transmission_stop()
{
	# disable transmission
	if [ "`pflash get btpdserv`" == "1" ] ; then
		pflash set btpdserv 0
	fi
	
	# check transmission is started
	if [ -n "`pidof transmission-daemon`" ] ; then
		# stop transmission
		/nmt/apps/server/clutch stop
	fi
	
	killall -q btpd_monitor
}

func_transmission_update()
{
	# update transmission-daemon to new version
	
	# backup native NMT transmission-daemon
	if [ ! -d "$bak_path" ] ; then
		mkdir -p "$bak_path/bin"
		mkdir -p "$bak_path/lib"
		mkdir -p "$bak_path/server"
		if [ -f /nmt/apps/server/clutch_backup ] ; then
			# use backup from CSI
			mv -f /nmt/apps/server/clutch_backup "$bak_path/server/clutch"
		else
			mv -f /nmt/apps/server/clutch "$bak_path/server"
		fi
		mv -f /nmt/apps/lib/libcurl.so.4 "$bak_path/lib"
		mv -f /nmt/apps/bin/transmission-daemon "$bak_path/bin"
		mv -f /nmt/apps/clutch5 "$bak_path"
	fi
	
	# delete native NMT transmission caller
	rm -f /nmt/apps/server/clutch
	
	# delete native NMT transmission-daemon
	rm -f /nmt/apps/bin/transmission-daemon
	
	# delete native NMT libraries
	rm -f /nmt/apps/lib/libcurl.so
	rm -f /nmt/apps/lib/libcurl.so.4
	
	# delete native NMT transmission web interface
	if [ -d /nmt/apps/clutch5 ] ; then
		rm -rf /nmt/apps/clutch5
	fi
	
	# copy new library
	cp -f "$dir_trmd/lib/libcurl.so.4" /nmt/apps/lib
	cp -f "$dir_trmd/lib/libcurl.so.4" /nmt/apps/lib/libcurl.so
	
	# copy new transmission-daemon
	cp -f "$dir_trmd/bin/transmission-daemon" /nmt/apps/bin
	
	# copy new transmission web interface
	cp -rf "$dir_trmd/clutch5" /nmt/apps
	
	# copy new NMT transmission caller
	cp -f "$dir_trmd/server/clutch" /nmt/apps/server
	cp -f "$dir_trmd/server/clutch" /nmt/apps/server/clutch_backup
	
	# try to copy torrents and settings from CSI Transmisson
	nmt_conf="/share/.transmission"
	if [ ! -f "$nmt_conf/.copied_from_csi" ] ; then
		mkdir -p -m 777 "$nmt_conf"
		csi_conf="/share/Apps/Transmission/.transmission"
		for i in "$csi_conf/settings.json" "$csi_conf/stats.json" "$csi_conf/dht.dat" ; do
			if  [ -f "$i" ] ; then
				cp -f "$i" "$nmt_conf"
				[ $? -eq 0 ] && echo "$i" >> "$nmt_conf/.copied_from_csi"
			fi
		done
		for i in "$csi_conf/resume" "$csi_conf/torrents" ; do
			if  [ -d "$i" ] ; then
				cp -r "$i" "$nmt_conf"
				[ $? -eq 0 ] && echo "$i" >> "$nmt_conf/.copied_from_csi"
			fi
		done
	fi
	
	if [ -f "$nmt_conf/settings.json" ] ; then
		chmod 777 "$nmt_conf/settings.json"
	fi
	
	# try to disable CSI Transmisson
	if [ -d /share/Apps/Transmission ] ; then
		/share/Apps/AppInit/appinit.cgi disable Transmission
	fi
}

func_transmission_restore()
{
	# restore transmission-daemon to native NMT version
	
	# restore from backup
	if [ -f "$bak_path/lib/libcurl.so.4" ] ; then
		cp -f "$bak_path/lib/libcurl.so.4" /nmt/apps/lib
		cp -f "$bak_path/lib/libcurl.so.4" /nmt/apps/lib/libcurl.so
	fi
	
	if [ -f "$bak_path/bin/transmission-daemon" ] ; then
		cp -f "$bak_path/bin/transmission-daemon" /nmt/apps/bin
	fi
	
	if [ -d "$bak_path/clutch5" ] ; then
		rm -rf /nmt/apps/clutch5
		cp -rf "$bak_path/clutch5" /nmt/apps
	fi
	
	if [ -f "$bak_path/server/clutch" ] ; then
		cp -f "$bak_path/server/clutch" /nmt/apps/server
		cp -f "$bak_path/server/clutch" /nmt/apps/server/clutch_backup
	fi
}

func_install()
{
	TERMINAL_SERVER=1
	PASSWD_ROOT="1234"
	PASSWD_NMT="1234"
	INSTALL_NANO=1
	INSTALL_MC=1
	INSTALL_SCREEN=1
	INSTALL_DISKUTILS=1
	INSTALL_E2FSPROGS=1
	INSTALL_TPFSPROGS=1
	START_LOGGER=1
	START_SMARTD=0
	TUNE_APACHE=1
	FIX_TCP_RMEM=1
	
	# load old settings
	if [ -f "$dir_this/configure" ] ; then
		. "$dir_this/configure"
	fi
	
	echo "
# set terminal server (0=off, 1=telnet, 2=ssh)
TERMINAL_SERVER=${TERMINAL_SERVER}

# set root password for telnet and ssh access (always required for ssh)
PASSWD_ROOT=\"${PASSWD_ROOT}\"

# set nmt password for telnet and ssh access (always required for ssh)
PASSWD_NMT=\"${PASSWD_NMT}\"

# install editor nano (terminal server needed)
INSTALL_NANO=${INSTALL_NANO}

# install file manager Midnight Commander (terminal server needed)
INSTALL_MC=${INSTALL_MC}

# install GNU screen (terminal server needed)
INSTALL_SCREEN=${INSTALL_SCREEN}

# install diskutils (fdisk, parted, hdparm, sdparm, blkid, lsusb, smartctl, smartd, transfer-test)
INSTALL_DISKUTILS=${INSTALL_DISKUTILS}

# install ext2/3 utils for check, create and tune EXT2, EXT3 filesystems
INSTALL_E2FSPROGS=${INSTALL_E2FSPROGS}

# install third-party utils for check FAT, FAT32, NTFS, HFS, HFS+ filesystems
INSTALL_TPFSPROGS=${INSTALL_TPFSPROGS}

# start system logger daemon (0=off, 1=log to tmpfs /tmp/messages, 2=log to drive /share/Apps/LinuxUtils/log/messages.all)
START_LOGGER=${START_LOGGER}

# start smartd logger daemon (logger needed)
START_SMARTD=${START_SMARTD}

# tune NMT Apache httpd.conf for reduce system overhead
TUNE_APACHE=${TUNE_APACHE}

# Fix TCP receive window for prevent kernel crash on bittorrent work
FIX_TCP_RMEM=${FIX_TCP_RMEM}" > "$dir_this/configure"
	
	chmod 777 "$dir_this/configure"
	
	if [ ! -f "$dir_this/user.script.sh" ] ; then
		echo "#!/bin/sh

#####################################################
### insert code here for run user custom commands ###
#####################################################

#####################################################
### Sample: tune first HDD drive via hdparam
#####################################################
### set spindown timer to 1 hour (241 = 30min, each next value add 30min)
### set APM to max (for stop Load Cycle Count on green drives)
### set AAM to quiet seek operations
#hdparm -S242 -B254 -M128 /dev/sda"> "$dir_this/user.script.sh"
		
		chmod 777 "$dir_this/user.script.sh"
	fi
	
	# copy ssh keys
	if [ ! -f "$dir_this/ssh/etc/dropbear/dss_key" ] ; then 
		cp "$dir_this/ssh/etc/dropbear/dss_key.def" "$dir_this/ssh/etc/dropbear/dss_key"
	fi
	
	if [ ! -f "$dir_this/ssh/etc/dropbear/rsa_key" ] ; then
		cp "$dir_this/ssh/etc/dropbear/rsa_key.def" "$dir_this/ssh/etc/dropbear/rsa_key"
	fi
	
	if [ ! -f "$dir_this/transmission/on_start.sh" ] ; then
		cp "$dir_this/transmission/default/.on_start.sh" "$dir_this/transmission/on_start.sh"
	fi
	
	if [ ! -f "$dir_this/transmission/on_stop.sh" ] ; then
		cp "$dir_this/transmission/default/.on_stop.sh" "$dir_this/transmission/on_stop.sh"
	fi
	
	# remove old placement
	rm -f "$dir_this/e2fsprogs/sbin/dosfsck"
	rm -f "/etc/mke2fs.conf"
	
	# unbind setups.cgi
	umount /opt/sybhttpd/default/setups.cgi 2>/dev/null
	umount /opt/sybhttpd/default/setups.cgi 2>/dev/null
}


case "$1" in
install)
	func_install
	func_transmission_stop
	func_transmission_update
	;;
uninstall)
	func_transmission_stop
	func_transmission_restore
	;;
btupdate)
	func_transmission_stop
	func_transmission_update
	;;
btrestore)
	func_transmission_stop
	func_transmission_restore
	;;
esac
